@@ -81,7 +81,7 @@ coordinates = np.array(
         [80 * np.cos((i + 1) / 4), 100 * np.sin((i + 1) / 4), 0]
     ] for i in range(1, 26 - 1)],
 )
-radii = [0.1 * (4 + i / 4) for i in range(1, 26 - 1)]
+radii = np.array([0.1 * (4 + i / 4) for i in range(1, 26 - 1)])[:, np.newaxis].repeat(3, 1)
 colors = np.array(
     [[0.8 * np.abs(np.cos(i / 8)), 0.8 * np.abs(np.cos((i + 3 * np.pi) / 8)), 0.8 * np.abs(np.cos((i + 6 * np.pi) / 8)), 0.8] for i in range(1, 26 - 1)],
 )
